`include "components/sn74vmeh22501.v"

`include "vme64x_bfm.svh"

module bidir_buf(
    a,
    b,
    dir, /* 0: a->b, 1: b->a */
    oe_n );

   parameter g_width = 1;
   
   inout [g_width-1:0] a,b;
   input               dir, oe_n;
   
   assign b = (!dir && !oe_n) ? a : 'bz;
   assign a = (dir && !oe_n) ? b : 'bz;
endmodule // bidir_buf




module svec_vme_buffers (
    output       VME_AS_n_o,
    output       VME_RST_n_o,
    output       VME_WRITE_n_o,
    output [5:0] VME_AM_o,
    output [1:0] VME_DS_n_o,
    output [5:0] VME_GA_o,
    input        VME_BERR_i,
    input        VME_DTACK_n_i,
    input        VME_RETRY_n_i,
    input        VME_RETRY_OE_i,
    inout        VME_LWORD_n_b,
    inout [31:1] VME_ADDR_b,
    inout [31:0] VME_DATA_b,
    output       VME_BBSY_n_o,
    input [6:0]  VME_IRQ_n_i,
//    output       VME_IACKIN_n_o,
//    input        VME_IACKOUT_n_i,
    output        VME_IACK_n_o,
    input        VME_DTACK_OE_i,
    input        VME_DATA_DIR_i,
    input        VME_DATA_OE_N_i,
    input        VME_ADDR_DIR_i,
    input        VME_ADDR_OE_N_i,
                 
    IVME64X.slave slave
    );

   pullup(slave.as_n);
   pullup(slave.rst_n);
   pullup(slave.irq_n[0]);
   pullup(slave.irq_n[1]);
   pullup(slave.irq_n[2]);
   pullup(slave.irq_n[3]);
   pullup(slave.irq_n[4]);
   pullup(slave.irq_n[5]);
   pullup(slave.irq_n[6]);
   pullup(slave.iack_n);
   pullup(slave.dtack_n);
   pullup(slave.retry_n);
   pullup(slave.ds_n[1]);
   pullup(slave.ds_n[0]);
   pullup(slave.lword_n);
   pullup(slave.berr_n);
   pullup(slave.write_n);
   pulldown(slave.bbsy_n);
//   pullup(slave.iackin_n);
//   pullup(slave.iackout_n);

   
   genvar        i;
   generate
      for(i=0;i<6;i++)
        assign slave.irq_n[i] = (VME_IRQ_n_i[i] ? 1'b0 : 1'bz);
   endgenerate
   
   
          
   
   assign VME_RST_n_o = slave.rst_n;
   
   assign VME_AS_n_o     = slave.as_n;
   assign VME_GA_o     = slave.ga;
   assign VME_WRITE_n_o  = slave.write_n;
   assign VME_AM_o = slave.am;
   assign VME_DS_n_o = slave.ds_n;
   assign VME_BBSY_n_o = slave.bbsy_n;
//   assign VME_IACKIN_n_o = slave.iackin_n;
   assign VME_IACK_n_o = slave.iack_n;

   bidir_buf #(1)  b0 (slave.lword_n, VME_LWORD_n_b, VME_ADDR_DIR_i, VME_ADDR_OE_N_i);
   bidir_buf #(31) b1 (slave.addr, VME_ADDR_b, VME_ADDR_DIR_i, VME_ADDR_OE_N_i);
   bidir_buf #(33) b2 (slave.data, VME_DATA_b, VME_DATA_DIR_i, VME_DATA_OE_N_i);

   pulldown(VME_BERR_i);
   pulldown(VME_ADDR_DIR_i);
   pulldown(VME_ADDR_OE_N_i);
   pulldown(VME_DATA_DIR_i);
   pulldown(VME_DATA_OE_N_i);
   


   
   assign slave.dtack_n = VME_DTACK_OE_i ? VME_DTACK_n_i : 1'bz;
   assign slave.berr_n = ~VME_BERR_i;
   assign slave.retry_n = VME_RETRY_n_i;
//   assign slave.iackout_n = VME_IACKOUT_n_i;
   
endmodule


`define DECLARE_VME_BUFFERS(iface,slot) \
    wire VME_AS_n_``slot;\
    wire VME_RST_n_``slot;\
    wire VME_WRITE_n_``slot;\
    wire [5:0] VME_AM_``slot;\
    wire [1:0] VME_DS_n_``slot;\
    wire VME_BERR_``slot;\
    wire VME_DTACK_n_``slot;\
    wire VME_RETRY_n_``slot;\
    wire VME_RETRY_OE_``slot;\
    wire VME_LWORD_n_``slot;\
    wire [31:1]VME_ADDR_``slot;\
    wire [31:0]VME_DATA_``slot;\
    wire VME_BBSY_n_``slot;\
    wire [6:0]VME_IRQ_n_``slot;\
    wire VME_IACK_n_``slot;\
//    wire VME_IACKOUT_n_``slot;\
    wire VME_DTACK_OE_``slot;\
    wire VME_DATA_DIR_``slot;\
    wire VME_DATA_OE_N_``slot;\
    wire VME_ADDR_DIR_``slot;\
    wire VME_ADDR_OE_N_``slot;\
    svec_vme_buffers U_VME_Bufs_``slot ( \
    .VME_AS_n_o(VME_AS_n_``slot),\
    .VME_RST_n_o(VME_RST_n_``slot),\
    .VME_WRITE_n_o(VME_WRITE_n_``slot),\
    .VME_AM_o(VME_AM_``slot),\
    .VME_DS_n_o(VME_DS_n_``slot),\
    .VME_BERR_i(VME_BERR_``slot),\
    .VME_DTACK_n_i(VME_DTACK_n_``slot),\
    .VME_RETRY_n_i(VME_RETRY_n_``slot),\
    .VME_RETRY_OE_i(VME_RETRY_OE_``slot),\
    .VME_LWORD_n_b(VME_LWORD_n_``slot),\
    .VME_ADDR_b(VME_ADDR_``slot),\
    .VME_DATA_b(VME_DATA_``slot),\
    .VME_BBSY_n_o(VME_BBSY_n_``slot),\
    .VME_IRQ_n_i(VME_IRQ_n_``slot),\
    .VME_IACK_n_o(VME_IACK_n_``slot),\
//   .VME_IACKIN_n_o(VME_IACKIN_n_``slot),\
//   .VME_IACKOUT_n_i(VME_IACKOUT_n_``slot),\
    .VME_DTACK_OE_i(VME_DTACK_OE_``slot),\
    .VME_DATA_DIR_i(VME_DATA_DIR_``slot),\
    .VME_DATA_OE_N_i(VME_DATA_OE_N_``slot),\
    .VME_ADDR_DIR_i(VME_ADDR_DIR_``slot),\
    .VME_ADDR_OE_N_i(VME_ADDR_OE_N_``slot),\
    .slave(iface)\
    );

function automatic bit[5:0] _gen_ga(int slot);
     bit[4:0] slot_id = slot;
     return {^slot_id, ~slot_id};
endfunction // _gen_ga

   
		
`define WIRE_VME_PINS(slot) \
    .VME_AS_n_i(VME_AS_n_``slot),\
    .VME_RST_n_i(VME_RST_n_``slot),\
    .VME_WRITE_n_i(VME_WRITE_n_``slot),\
    .VME_AM_i(VME_AM_``slot),\
    .VME_DS_n_i(VME_DS_n_``slot),\
    .VME_GA_i(_gen_ga(slot)),\
    .VME_BERR_o(VME_BERR_``slot),\
    .VME_DTACK_n_o(VME_DTACK_n_``slot),\
    .VME_RETRY_n_o(VME_RETRY_n_``slot),\
    .VME_RETRY_OE_o(VME_RETRY_OE_``slot),\
    .VME_LWORD_n_b(VME_LWORD_n_``slot),\
    .VME_ADDR_b(VME_ADDR_``slot),\
    .VME_DATA_b(VME_DATA_``slot),\
    .VME_BBSY_n_i(VME_BBSY_n_``slot),\
    .VME_IRQ_n_o(VME_IRQ_n_``slot),\
    .VME_IACK_n_i(VME_IACK_n_``slot),\
//    .VME_IACKIN_n_i(VME_IACKIN_n_``slot),\
//    .VME_IACKOUT_n_o(VME_IACKOUT_n_``slot),\
    .VME_DTACK_OE_o(VME_DTACK_OE_``slot),\
    .VME_DATA_DIR_o(VME_DATA_DIR_``slot),\
    .VME_DATA_OE_N_o(VME_DATA_OE_N_``slot),\
    .VME_ADDR_DIR_o(VME_ADDR_DIR_``slot),\
    .VME_ADDR_OE_N_o(VME_ADDR_OE_N_``slot)


		