`include "vme64x_bfm.svh"
`include "svec_vme_buffers.svh"

`include "fdelay_board.svh"


module main;

   reg rst_n = 0;
   reg clk_125m = 0, clk_20m = 0;

   always #4ns clk_125m <= ~clk_125m;
   always #25ns clk_20m <= ~clk_20m;
   
   initial begin
      repeat(20) @(posedge clk_125m);
      rst_n = 1;
   end

   //IFineDelayFMC I_fmc0(), I_fmc1();
   
   IVME64X VME(rst_n);

   `DECLARE_VME_BUFFERS(VME.slave,8);
   `DECLARE_VME_BUFFERS(VME.slave,9);
   `DECLARE_VME_BUFFERS(VME.slave,10);

   wire daisy_iack,daisy_iack2,daisy_iack3;
   
   
   svec_top #(
              .g_with_wr_phy(0),
              .g_simulation(1)
              ) U_SVEC1 (
		         .clk_125m_pllref_p_i(clk_125m),
		         .clk_125m_pllref_n_i(~clk_125m),
		         .clk_125m_gtp_p_i(clk_125m),
		         .clk_125m_gtp_n_i(~clk_125m),
		         .clk_20m_vcxo_i(clk_20m),
                         
		         .rst_n_i(rst_n),
                         .VME_IACKIN_n_i(VME.iack_n),
//                         .VME_IACK_n_i(VME.iack_n),
                         .VME_IACKOUT_n_o(daisy_iack),
                         
		         `WIRE_VME_PINS(8)
      
	                 );

      svec_top #(
              .g_with_wr_phy(0),
              .g_simulation(1)
              ) U_SVEC2 (
		 .clk_125m_pllref_p_i(clk_125m),
		 .clk_125m_pllref_n_i(~clk_125m),
		 .clk_125m_gtp_p_i(clk_125m),
		 .clk_125m_gtp_n_i(~clk_125m),
		 .clk_20m_vcxo_i(clk_20m),
                
		 .rst_n_i(rst_n),

                 .VME_IACKIN_n_i(daisy_iack),
  //               .VME_IACK_n_i(VME.iack_n),
                 .VME_IACKOUT_n_o(daisy_iack2),
                 
		 `WIRE_VME_PINS(9)
	         );
     
   svec_top #(
              .g_with_wr_phy(0),
              .g_simulation(1)
              ) U_SVEC3 (
		 .clk_125m_pllref_p_i(clk_125m),
		 .clk_125m_pllref_n_i(~clk_125m),
		 .clk_125m_gtp_p_i(clk_125m),
		 .clk_125m_gtp_n_i(~clk_125m),
		 .clk_20m_vcxo_i(clk_20m),
                
		 .rst_n_i(rst_n),

                 .VME_IACKIN_n_i(daisy_iack2),
  //               .VME_IACK_n_i(VME.iack_n),
                 .VME_IACKOUT_n_o(daisy_iack3),
                 
		 `WIRE_VME_PINS(10)
	         );

    task automatic config_vme_function(ref CBusAccessor_VME64x acc, input int slot, input int func, uint64_t base, int am);
      uint64_t addr = 'h7ff63 + func * 'h10;
      uint64_t val = (base) | (am << 2);

       acc.set_slot(slot);
       
      $display("Func%d ADER=0x%x", func, val);
      
      
      acc.write(addr + 0, (val >> 24) & 'hff, CR_CSR|A32|D08Byte3);
      acc.write(addr + 4, (val >> 16) & 'hff, CR_CSR|A32|D08Byte3);
      acc.write(addr + 8, (val >> 8)  & 'hff, CR_CSR|A32|D08Byte3);
      acc.write(addr + 12, (val >> 0) & 'hff, CR_CSR|A32|D08Byte3);
 
      
   endtask // config_vme_function

   task automatic enable_vme_core (ref CBusAccessor_VME64x acc, input int slot );
      acc.set_slot(slot);
      acc.write('h7ff33, 1, CR_CSR|A32|D08Byte3);
      acc.write('h7fffb, 'h10, CR_CSR|A32|D08Byte3); /* enable module (BIT_SET = 0x10) */

      acc.set_default_modifiers(A24 | D32 | SINGLE);
   
   endtask // enable_vme_core
   
   
   task automatic init_vme64x_core(ref CBusAccessor_VME64x acc);
      uint64_t rv;


      /* map slot 8 func1 to 0xc00000, A24 */
      config_vme_function(acc, 8, 1, 'hc00000, 'h39);
      enable_vme_core(acc, 8);

      config_vme_function(acc, 9, 1, 'hc80000, 'h39);
      enable_vme_core(acc, 9);

      acc.set_default_modifiers(A24 | D32 | SINGLE);
   endtask // init_vme64x_core
   

   reg irq_req = 0;



   initial begin
      #100us;
      irq_req = 1;
      #100us;
      irq_req = 0;
      #100us;
      irq_req = 0;
      #1ms;
      irq_req = 0;
      #1ms;
      
      
   end
  
  
   assign  U_SVEC1.vic_master_irq = irq_req;
   assign  U_SVEC2.vic_master_irq = 0;
   assign  U_SVEC3.vic_master_irq = 0;

   initial begin
      uint64_t rv;
      
      CBusAccessor_VME64x acc = new(VME.master);
      CBusAccessor acc_casted = CBusAccessor'(acc);
      
      #50us;

      init_vme64x_core(acc);
//      init_vme64x_core(acc, 9, 'hb80000, 'h88);
    
      acc_casted.set_default_xfer_size(A24|SINGLE|D32);
      
      acc.read('hc00000, rv, A24|SINGLE|D32);
      acc.read('hc80000, rv, A24|SINGLE|D32);
//      acc.read('hb80000, rv, A24|SINGLE|D32);
      
   end // initial begin
   



   
endmodule // main




